@@ -7,16 +7,19 @@
 #include "nsCOMPtr.h"
 #include "mozIJSSubScriptLoader.h"
 
+#include "js/experimental/JSStencil.h"
+#include "js/CompileOptions.h"  // JS::ReadOnlyCompileOptions
+
 class nsIPrincipal;
 class nsIURI;
 class LoadSubScriptOptions;
 
-#define MOZ_JSSUBSCRIPTLOADER_CID                    \
-  { /* 829814d6-1dd2-11b2-8e08-82fa0a339b00 */       \
-    0x929814d6, 0x1dd2, 0x11b2, {                    \
-      0x8e, 0x08, 0x82, 0xfa, 0x0a, 0x33, 0x9b, 0x00 \
-    }                                                \
-  }
+#define MOZ_JSSUBSCRIPTLOADER_CID             \
+  {/* 829814d6-1dd2-11b2-8e08-82fa0a339b00 */ \
+   0x929814d6,                                \
+   0x1dd2,                                    \
+   0x11b2,                                    \
+   {0x8e, 0x08, 0x82, 0xfa, 0x0a, 0x33, 0x9b, 0x00}}
 
 class nsIIOService;
 
@@ -31,10 +34,9 @@ class mozJSSubScriptLoader : public mozIJSSubScriptLoader {
  private:
   virtual ~mozJSSubScriptLoader();
 
-  bool ReadScript(JS::MutableHandle<JSScript*> script,
-                  nsIURI* uri, JSContext* cx, JS::HandleObject targetObj,
-                  const char* uriStr, nsIIOService* serv,
-                  bool wantReturnValue, bool useCompilationScope);
+  bool ReadStencil(JS::Stencil** stencilOut, nsIURI* uri, JSContext* cx,
+                   const JS::ReadOnlyCompileOptions& options,
+                   nsIIOService* serv, bool useCompilationScope);
 
   nsresult ReadScriptAsync(nsIURI* uri, JS::HandleObject targetObj,
                            JS::HandleObject loadScope, nsIIOService* serv,
