@@ -22,6 +22,7 @@
 #include "mozilla/dom/ScriptSettings.h"
 #include "mozilla/dom/MaybeCrossOriginObject.h"
 #include "nsContentUtils.h"
+#include "nsGlobalWindowInner.h"
 #include "nsXULAppAPI.h"
 
 using namespace JS;
@@ -30,6 +31,13 @@ using namespace mozilla;
 
 namespace xpc {
 
+#ifndef MOZ_UNIFIED_BUILD
+extern template class FilteringWrapper<js::CrossCompartmentSecurityWrapper,
+                                       Opaque>;
+extern template class FilteringWrapper<js::CrossCompartmentSecurityWrapper,
+                                       OpaqueWithCall>;
+#endif
+
 // When chrome pulls a naked property across the membrane using
 // .wrappedJSObject, we want it to cross the membrane into the
 // chrome compartment without automatically being wrapped into an
@@ -235,7 +243,7 @@ void WrapperFactory::PrepareForWrapping(JSContext* cx, HandleObject scope,
   // those objects in a security wrapper, then we need to hand back the
   // wrapper for the new scope instead. Also, global objects don't move
   // between scopes so for those we also want to return the wrapper. So...
-  if (!IS_WN_REFLECTOR(obj) || JS_IsGlobalObject(obj)) {
+  if (!IsWrappedNativeReflector(obj) || JS_IsGlobalObject(obj)) {
     retObj.set(waive ? WaiveXray(cx, obj) : obj);
     return;
   }
@@ -266,7 +274,7 @@ void WrapperFactory::PrepareForWrapping(JSContext* cx, HandleObject scope,
     // that we don't create a second JS object for it: create a security
     // wrapper.
     //
-    // Note: The only two objects that still use PreCreate are BackstagePass
+    // Note: The only two objects that still use PreCreate are SystemGlobal
     // and Components, both of which unconditionally request their canonical
     // scope. Since SpiderMonkey only invokes the prewrap callback in
     // situations where the object is nominally cross-compartment, we should
@@ -288,7 +296,7 @@ void WrapperFactory::PrepareForWrapping(JSContext* cx, HandleObject scope,
   }
 
   obj.set(&v.toObject());
-  MOZ_ASSERT(IS_WN_REFLECTOR(obj), "bad object");
+  MOZ_ASSERT(IsWrappedNativeReflector(obj), "bad object");
   JS::AssertObjectIsNotGray(obj);  // We should never return gray reflectors.
 
   // Because the underlying native didn't have a PreCreate hook, we had
@@ -385,7 +393,8 @@ static void DEBUG_CheckUnwrapSafety(HandleObject obj,
 }
 #else
 #  define DEBUG_CheckUnwrapSafety(obj, handler, origin, target) \
-    {}
+    {                                                           \
+    }
 #endif
 
 const CrossOriginObjectWrapper CrossOriginObjectWrapper::singleton;
@@ -478,7 +487,8 @@ JSObject* WrapperFactory::Rewrap(JSContext* cx, HandleObject existing,
   if (originIsChrome && !targetIsChrome) {
     // If this is a chrome function being exposed to content, we need to allow
     // call (but nothing else).
-    if ((IdentifyStandardInstance(obj) == JSProto_Function)) {
+    JSProtoKey key = IdentifyStandardInstance(obj);
+    if (key == JSProto_Function || key == JSProto_BoundFunction) {
       wrapper = &FilteringWrapper<CrossCompartmentSecurityWrapper,
                                   OpaqueWithCall>::singleton;
     }
@@ -486,7 +496,7 @@ JSObject* WrapperFactory::Rewrap(JSContext* cx, HandleObject existing,
     // For vanilla JSObjects exposed from chrome to content, we use a wrapper
     // that fails silently in a few cases. We'd like to get rid of this
     // eventually, but in their current form they don't cause much trouble.
-    else if (IdentifyStandardInstance(obj) == JSProto_Object) {
+    else if (key == JSProto_Object) {
       wrapper = &ChromeObjectWrapper::singleton;
     }
 
@@ -788,7 +798,7 @@ nsIGlobalObject* NativeGlobal(JSObject* obj) {
     MOZ_ASSERT(native);
 
     // In some cases (like for windows) it is a wrapped native,
-    // in other cases (sandboxes, backstage passes) it's just
+    // in other cases (sandboxes, system globals) it's just
     // a direct pointer to the native. If it's a wrapped native
     // let's unwrap it first.
     if (nsCOMPtr<nsIXPConnectWrappedNative> wn = do_QueryInterface(native)) {
