@@ -6,27 +6,27 @@
 const { CryptoUtils } = ChromeUtils.importESModule(
   "resource://services-crypto/utils.sys.mjs"
 );
-const { FxAccounts } = ChromeUtils.importESModule(
+const { FxAccounts, ERROR_INVALID_ACCOUNT_STATE } = ChromeUtils.importESModule(
   "resource://gre/modules/FxAccounts.sys.mjs"
 );
 const { FxAccountsClient } = ChromeUtils.importESModule(
   "resource://gre/modules/FxAccountsClient.sys.mjs"
 );
 const {
+  CLIENT_IS_THUNDERBIRD,
   ERRNO_INVALID_AUTH_TOKEN,
   ERROR_NO_ACCOUNT,
-  FX_OAUTH_CLIENT_ID,
+  OAUTH_CLIENT_ID,
   ONLOGIN_NOTIFICATION,
   ONLOGOUT_NOTIFICATION,
   ONVERIFIED_NOTIFICATION,
   DEPRECATED_SCOPE_ECOSYSTEM_TELEMETRY,
   PREF_LAST_FXA_USER,
-} = ChromeUtils.import("resource://gre/modules/FxAccountsCommon.js");
-const { PromiseUtils } = ChromeUtils.importESModule(
-  "resource://gre/modules/PromiseUtils.sys.mjs"
+} = ChromeUtils.importESModule(
+  "resource://gre/modules/FxAccountsCommon.sys.mjs"
 );
 
-// We grab some additional stuff via backstage passes.
+// We grab some additional stuff via the system global.
 var { AccountState } = ChromeUtils.importESModule(
   "resource://gre/modules/FxAccounts.sys.mjs"
 );
@@ -35,7 +35,7 @@ const MOCK_TOKEN_RESPONSE = {
   access_token:
     "43793fdfffec22eb39fc3c44ed09193a6fde4c24e5d6a73f73178597b268af69",
   token_type: "bearer",
-  scope: "https://identity.mozilla.com/apps/oldsync",
+  scope: SCOPE_APP_SYNC,
   expires_in: 21600,
   auth_at: 1589579900,
 };
@@ -45,10 +45,10 @@ initTestLogging("Trace");
 var log = Log.repository.getLogger("Services.FxAccounts.test");
 log.level = Log.Level.Debug;
 
-// See verbose logging from FxAccounts.jsm and jwcrypto.jsm.
-Services.prefs.setCharPref("identity.fxaccounts.loglevel", "Trace");
+// See verbose logging from FxAccounts.sys.mjs and jwcrypto.sys.mjs.
+Services.prefs.setStringPref("identity.fxaccounts.loglevel", "Trace");
 Log.repository.getLogger("FirefoxAccounts").level = Log.Level.Trace;
-Services.prefs.setCharPref("services.crypto.jwcrypto.log.level", "Debug");
+Services.prefs.setStringPref("services.crypto.jwcrypto.log.level", "Debug");
 
 /*
  * The FxAccountsClient communicates with the remote Firefox
@@ -121,7 +121,7 @@ function MockFxAccountsClient() {
 
   // mock calls up to the auth server to determine whether the
   // user account has been verified
-  this.recoveryEmailStatus = async function (sessionToken) {
+  this.recoveryEmailStatus = async function () {
     // simulate a call to /recovery_email/status
     return {
       email: this._email,
@@ -140,7 +140,7 @@ function MockFxAccountsClient() {
     return !this._deletedOnServer;
   };
 
-  this.accountKeys = function (keyFetchToken) {
+  this.accountKeys = function () {
     return new Promise(resolve => {
       do_timeout(50, () => {
         resolve({
@@ -153,13 +153,13 @@ function MockFxAccountsClient() {
 
   this.getScopedKeyData = function (sessionToken, client_id, scopes) {
     Assert.ok(sessionToken);
-    Assert.equal(client_id, FX_OAUTH_CLIENT_ID);
-    Assert.equal(scopes, SCOPE_OLD_SYNC);
+    Assert.equal(client_id, OAUTH_CLIENT_ID);
+    Assert.equal(scopes, SCOPE_APP_SYNC);
     return new Promise(resolve => {
       do_timeout(50, () => {
         resolve({
-          "https://identity.mozilla.com/apps/oldsync": {
-            identifier: "https://identity.mozilla.com/apps/oldsync",
+          [SCOPE_APP_SYNC]: {
+            identifier: SCOPE_APP_SYNC,
             keyRotationSecret:
               "0000000000000000000000000000000000000000000000000000000000000000",
             keyRotationTimestamp: 1234567890123,
@@ -189,12 +189,12 @@ Object.setPrototypeOf(
  * mock the now() method, so that we can simulate the passing of
  * time and verify that signatures expire correctly.
  */
-function MockFxAccounts(credentials = null) {
+function MockFxAccounts() {
   let result = new FxAccounts({
     VERIFICATION_POLL_TIMEOUT_INITIAL: 100, // 100ms
 
     _getCertificateSigned_calls: [],
-    _d_signCertificate: PromiseUtils.defer(),
+    _d_signCertificate: Promise.withResolvers(),
     _now_is: new Date(),
     now() {
       return this._now_is;
@@ -218,7 +218,7 @@ function MockFxAccounts(credentials = null) {
     },
   });
   // and for convenience so we don't have to touch as many lines in this test
-  // when we refactored FxAccounts.jsm :)
+  // when we refactored FxAccounts.sys.mjs :)
   result.setSignedInUser = function (creds) {
     return result._internal.setSignedInUser(creds);
   };
@@ -282,18 +282,11 @@ add_task(async function test_get_signed_in_user_initially_unset() {
   result = await account.getSignedInUser();
   Assert.deepEqual(result.email, credentials.email);
   Assert.deepEqual(result.scopedKeys, undefined);
-  Assert.deepEqual(result.kSync, undefined);
-  Assert.deepEqual(result.kXCS, undefined);
-  Assert.deepEqual(result.kExtSync, undefined);
-  Assert.deepEqual(result.kExtKbHash, undefined);
+
   // for the sake of testing, use the low-level function to check it's all there
   result = await account._internal.currentAccountState.getUserAccountData();
   Assert.deepEqual(result.email, credentials.email);
   Assert.deepEqual(result.scopedKeys, credentials.scopedKeys);
-  Assert.ok(result.kSync);
-  Assert.ok(result.kXCS);
-  Assert.ok(result.kExtSync);
-  Assert.ok(result.kExtKbHash);
 
   // sign out
   let localOnly = true;
@@ -399,6 +392,7 @@ add_test(function test_client_mock() {
 // Polling should detect that the email is verified, and eventually
 // 'onverified' should be observed
 add_test(function test_verification_poll() {
+  ensureOauthNotConfigured();
   let fxa = new MockFxAccounts();
   let test_user = getTestUser("francine");
   let login_notification_received = false;
@@ -443,6 +437,7 @@ add_test(function test_verification_poll() {
 // poll should time out.  No verifiedlogin event should be observed, and the
 // internal whenVerified promise should be rejected
 add_test(function test_polling_timeout() {
+  ensureOauthNotConfigured();
   // This test could be better - the onverified observer might fire on
   // somebody else's stack, and we're not making sure that we're not receiving
   // such a message. In other words, this tests either failure, or success, but
@@ -461,10 +456,10 @@ add_test(function test_polling_timeout() {
 
   fxa.setSignedInUser(test_user).then(() => {
     p.then(
-      success => {
+      () => {
         do_throw("this should not succeed");
       },
-      fail => {
+      () => {
         removeObserver();
         fxa.signOut().then(run_next_test);
       }
@@ -479,7 +474,7 @@ add_task(async function test_onverified_once() {
 
   let numNotifications = 0;
 
-  function observe(aSubject, aTopic, aData) {
+  function observe() {
     numNotifications += 1;
   }
   Services.obs.addObserver(observe, ONVERIFIED_NOTIFICATION);
@@ -682,24 +677,15 @@ add_test(function test_getKeyForScope() {
     fxa._internal.getUserAccountData().then(user2 => {
       // Before getKeyForScope, we have no keys
       Assert.equal(!!user2.scopedKeys, false);
-      Assert.equal(!!user2.kSync, false);
-      Assert.equal(!!user2.kXCS, false);
-      Assert.equal(!!user2.kExtSync, false);
-      Assert.equal(!!user2.kExtKbHash, false);
       // And we still have a key-fetch token and unwrapBKey to use
       Assert.equal(!!user2.keyFetchToken, true);
       Assert.equal(!!user2.unwrapBKey, true);
 
-      fxa.keys.getKeyForScope(SCOPE_OLD_SYNC).then(() => {
+      fxa.keys.getKeyForScope(SCOPE_APP_SYNC).then(() => {
         fxa._internal.getUserAccountData().then(user3 => {
           // Now we should have keys
-          Assert.equal(fxa._internal.isUserEmailVerified(user3), true);
           Assert.equal(!!user3.verified, true);
           Assert.notEqual(null, user3.scopedKeys);
-          Assert.notEqual(null, user3.kSync);
-          Assert.notEqual(null, user3.kXCS);
-          Assert.notEqual(null, user3.kExtSync);
-          Assert.notEqual(null, user3.kExtKbHash);
           Assert.equal(user3.keyFetchToken, undefined);
           Assert.equal(user3.unwrapBKey, undefined);
           run_next_test();
@@ -709,82 +695,58 @@ add_test(function test_getKeyForScope() {
   });
 });
 
-add_task(async function test_getKeyForScope_kb_migration() {
+add_task(async function test_oauth_verification() {
+  ensureOauthConfigured();
   let fxa = new MockFxAccounts();
   let user = getTestUser("eusebius");
-
   user.verified = true;
-  // Set-up the deprecated set of keys.
-  user.kA = "e0245ab7f10e483470388e0a28f0a03379a3b417174fb2b42feab158b4ac2dbd";
-  user.kB = "eaf9570b7219a4187d3d6bf3cec2770c2e0719b7cc0dfbb38243d6f1881675e9";
 
   await fxa.setSignedInUser(user);
-  await fxa.keys.getKeyForScope(SCOPE_OLD_SYNC);
-  let newUser = await fxa._internal.getUserAccountData();
-  Assert.equal(newUser.kA, null);
-  Assert.equal(newUser.kB, null);
-  Assert.deepEqual(newUser.scopedKeys, {
-    "https://identity.mozilla.com/apps/oldsync": {
-      kid: "1234567890123-IqQv4onc7VcVE1kTQkyyOw",
-      k: "DW_ll5GwX6SJ5GPqJVAuMUP2t6kDqhUulc2cbt26xbTcaKGQl-9l29FHAQ7kUiJETma4s9fIpEHrt909zgFang",
-      kty: "oct",
-    },
-    "sync:addon_storage": {
-      kid: "1234567890123-pBOR6B6JulbJr3BxKVOqIU4Cq_WAjFp4ApLn5NRVARE",
-      k: "ut7VPrNYfXkA5gTopo2GCr-d4wtclV08TV26Y_Jv2IJlzYWSP26dzRau87gryIA5qJxZ7NnojeCadBjH2U-QyQ",
-      kty: "oct",
-    },
+  let fetched = await fxa.getSignedInUser();
+  Assert.ok(!fetched.verified);
+
+  fxa._withCurrentAccountState(state => {
+    state.updateUserAccountData({ scopedKeys: { test: { foo: "bar" } } });
   });
-  // These hex values were manually confirmed to be equivalent to the b64 values above.
-  Assert.equal(
-    newUser.kSync,
-    "0d6fe59791b05fa489e463ea25502e3143f6b7a903aa152e95cd9c6eddbac5b4" +
-      "dc68a19097ef65dbd147010ee45222444e66b8b3d7c8a441ebb7dd3dce015a9e"
-  );
-  Assert.equal(newUser.kXCS, "22a42fe289dced5715135913424cb23b");
-  Assert.equal(
-    newUser.kExtSync,
-    "baded53eb3587d7900e604e8a68d860abf9de30b5c955d3c4d5dba63f26fd882" +
-      "65cd85923f6e9dcd16aef3b82bc88039a89c59ecd9e88de09a7418c7d94f90c9"
-  );
-  Assert.equal(
-    newUser.kExtKbHash,
-    "a41391e81e89ba56c9af70712953aa214e02abf5808c5a780292e7e4d4550111"
-  );
+
+  fetched = await fxa.getSignedInUser();
+  Assert.ok(fetched.verified);
+
+  resetOauthConfig();
 });
 
-add_task(async function test_getKeyForScope_scopedKeys_migration() {
-  let fxa = new MockFxAccounts();
-  let user = getTestUser("eusebius");
+add_task(
+  async function test_getKeyForScope_scopedKeys_migration_removes_deprecated_high_level_keys() {
+    let fxa = new MockFxAccounts();
+    let user = getTestUser("eusebius");
 
-  user.verified = true;
-  // Set-up the keys in deprecated fields.
-  user.kSync = MOCK_ACCOUNT_KEYS.kSync;
-  user.kXCS = MOCK_ACCOUNT_KEYS.kXCS;
-  user.kExtSync = MOCK_ACCOUNT_KEYS.kExtSync;
-  user.kExtKbHash = MOCK_ACCOUNT_KEYS.kExtKbHash;
-  Assert.equal(user.scopedKeys, null);
+    user.verified = true;
 
-  await fxa.setSignedInUser(user);
-  await fxa.keys.getKeyForScope(SCOPE_OLD_SYNC);
-  let newUser = await fxa._internal.getUserAccountData();
-  Assert.equal(newUser.kA, null);
-  Assert.equal(newUser.kB, null);
-  // It should have correctly formatted the corresponding scoped keys.
-  const expectedScopedKeys = { ...MOCK_ACCOUNT_KEYS.scopedKeys };
-  Assert.deepEqual(newUser.scopedKeys, expectedScopedKeys);
-  // And left the existing key fields unchanged.
-  Assert.equal(newUser.kSync, user.kSync);
-  Assert.equal(newUser.kXCS, user.kXCS);
-  Assert.equal(newUser.kExtSync, user.kExtSync);
-  Assert.equal(newUser.kExtKbHash, user.kExtKbHash);
-});
+    // An account state with the deprecated kinto extension sync keys...
+    user.kExtSync =
+      "f5ccd9cfdefd9b1ac4d02c56964f59239d8dfa1ca326e63696982765c1352cdc" +
+      "5d78a5a9c633a6d25edfea0a6c221a3480332a49fd866f311c2e3508ddd07395";
+    user.kExtKbHash =
+      "6192f1cc7dce95334455ba135fa1d8fca8f70e8f594ae318528de06f24ed0273";
+    user.scopedKeys = {
+      ...MOCK_ACCOUNT_KEYS.scopedKeys,
+    };
+
+    await fxa.setSignedInUser(user);
+    // getKeyForScope will run the migration
+    await fxa.keys.getKeyForScope(SCOPE_APP_SYNC);
+    let newUser = await fxa._internal.getUserAccountData();
+    // Then, the deprecated keys will be removed
+    Assert.strictEqual(newUser.kExtSync, undefined);
+    Assert.strictEqual(newUser.kExtKbHash, undefined);
+  }
+);
 
 add_task(
   async function test_getKeyForScope_scopedKeys_migration_removes_deprecated_scoped_keys() {
     let fxa = new MockFxAccounts();
     let user = getTestUser("eusebius");
-
+    const DEPRECATED_SCOPE_WEBEXT_SYNC = "sync:addon_storage";
     const EXTRA_SCOPE = "an unknown, but non-deprecated scope";
     user.verified = true;
     user.ecosystemUserId = "ecoUserId";
@@ -792,18 +754,21 @@ add_task(
     user.scopedKeys = {
       ...MOCK_ACCOUNT_KEYS.scopedKeys,
       [DEPRECATED_SCOPE_ECOSYSTEM_TELEMETRY]:
-        MOCK_ACCOUNT_KEYS.scopedKeys[SCOPE_OLD_SYNC],
-      [EXTRA_SCOPE]: MOCK_ACCOUNT_KEYS.scopedKeys[SCOPE_OLD_SYNC],
+        MOCK_ACCOUNT_KEYS.scopedKeys[SCOPE_APP_SYNC],
+      [DEPRECATED_SCOPE_WEBEXT_SYNC]:
+        MOCK_ACCOUNT_KEYS.scopedKeys[SCOPE_APP_SYNC],
+      [EXTRA_SCOPE]: MOCK_ACCOUNT_KEYS.scopedKeys[SCOPE_APP_SYNC],
     };
 
     await fxa.setSignedInUser(user);
-    await fxa.keys.getKeyForScope(SCOPE_OLD_SYNC);
+    await fxa.keys.getKeyForScope(SCOPE_APP_SYNC);
     let newUser = await fxa._internal.getUserAccountData();
     // It should have removed the deprecated ecosystem_telemetry key,
+    // and the old kinto extension sync key
     // but left the other keys intact.
     const expectedScopedKeys = {
       ...MOCK_ACCOUNT_KEYS.scopedKeys,
-      [EXTRA_SCOPE]: MOCK_ACCOUNT_KEYS.scopedKeys[SCOPE_OLD_SYNC],
+      [EXTRA_SCOPE]: MOCK_ACCOUNT_KEYS.scopedKeys[SCOPE_APP_SYNC],
     };
     Assert.deepEqual(newUser.scopedKeys, expectedScopedKeys);
     Assert.equal(newUser.ecosystemUserId, null);
@@ -834,11 +799,10 @@ add_task(async function test_getKeyForScope_nonexistent_account() {
     });
   });
 
-  // XXX - the exception message here isn't ideal, but doesn't really matter...
-  await Assert.rejects(
-    fxa.keys.getKeyForScope(SCOPE_OLD_SYNC),
-    /A different user signed in/
-  );
+  await Assert.rejects(fxa.keys.getKeyForScope(SCOPE_APP_SYNC), err => {
+    Assert.equal(err.message, ERROR_INVALID_ACCOUNT_STATE);
+    return true; // expected error
+  });
 
   await promiseLogout;
 
@@ -866,7 +830,7 @@ add_task(async function test_getKeyForScope_invalid_token() {
   Assert.notEqual(user.encryptedSendTabKeys, null);
 
   try {
-    await fxa.keys.getKeyForScope(SCOPE_OLD_SYNC);
+    await fxa.keys.getKeyForScope(SCOPE_APP_SYNC);
     Assert.ok(false);
   } catch (err) {
     Assert.equal(err.code, 401);
@@ -882,36 +846,77 @@ add_task(async function test_getKeyForScope_invalid_token() {
   await fxa._internal.abortExistingFlow();
 });
 
-// This is the exact same test vectors as
-// https://github.com/mozilla/fxa-crypto-relier/blob/f94f441159029a645a474d4b6439c38308da0bb0/test/deriver/ScopedKeys.js#L58
+// Test vectors from
+// https://wiki.mozilla.org/Identity/AttachedServices/KeyServerProtocol#Test_Vectors
 add_task(async function test_getKeyForScope_oldsync() {
   let fxa = new MockFxAccounts();
   let client = fxa._internal.fxAccountsClient;
   client.getScopedKeyData = () =>
     Promise.resolve({
-      "https://identity.mozilla.com/apps/oldsync": {
-        identifier: "https://identity.mozilla.com/apps/oldsync",
+      [SCOPE_APP_SYNC]: {
+        identifier: SCOPE_APP_SYNC,
         keyRotationSecret:
           "0000000000000000000000000000000000000000000000000000000000000000",
         keyRotationTimestamp: 1510726317123,
       },
     });
+
+  // We mock the server returning the wrapKB from our test vectors
+  client.accountKeys = async () => {
+    return {
+      wrapKB: CommonUtils.hexToBytes(
+        "404142434445464748494a4b4c4d4e4f505152535455565758595a5b5c5d5e5f"
+      ),
+    };
+  };
+
+  // We set the user to have the keyFetchToken and unwrapBKey from our test vectors
   let user = {
     ...getTestUser("eusebius"),
     uid: "aeaa1725c7a24ff983c6295725d5fc9b",
-    kB: "eaf9570b7219a4187d3d6bf3cec2770c2e0719b7cc0dfbb38243d6f1881675e9",
+    keyFetchToken:
+      "808182838485868788898a8b8c8d8e8f909192939495969798999a9b9c9d9e9f",
+    unwrapBKey:
+      "6ea660be9c89ec355397f89afb282ea0bf21095760c8c5009bbcc894155bbe2a",
+    sessionToken: "mock session token, used in metadata request",
     verified: true,
   };
   await fxa.setSignedInUser(user);
-  const key = await fxa.keys.getKeyForScope(SCOPE_OLD_SYNC);
+
+  // We derive, persist and return the sync key
+  const key = await fxa.keys.getKeyForScope(SCOPE_APP_SYNC);
+
+  // We verify the key returned matches what we would expect from the test vectors
+  // kb = 2ee722fdd8ccaa721bdeb2d1b76560efef705b04349d9357c3e592cf4906e075 (from test vectors)
+  //
+  // kid can be verified by "${keyRotationTimestamp}-${sha256(kb)[0:16]}"
+  //
+  // k can be verified by HKDF(kb, undefined, "identity.mozilla.com/picl/v1/oldsync", 64)
   Assert.deepEqual(key, {
-    scope: SCOPE_OLD_SYNC,
-    kid: "1510726317123-IqQv4onc7VcVE1kTQkyyOw",
-    k: "DW_ll5GwX6SJ5GPqJVAuMUP2t6kDqhUulc2cbt26xbTcaKGQl-9l29FHAQ7kUiJETma4s9fIpEHrt909zgFang",
+    scope: SCOPE_APP_SYNC,
+    kid: "1510726317123-BAik7hEOdpGnPZnPBSdaTg",
+    k: "fwM5VZu0Spf5XcFRZYX2zk6MrqZP7zvovCBcvuKwgYMif3hz98FHmIVa3qjKjrW0J244Zj-P5oWaOcQbvypmpw",
     kty: "oct",
   });
 });
 
+add_task(async function test_getScopedKeys_cached_key() {
+  let fxa = new MockFxAccounts();
+  let user = {
+    ...getTestUser("eusebius"),
+    uid: "aeaa1725c7a24ff983c6295725d5fc9b",
+    verified: true,
+    ...MOCK_ACCOUNT_KEYS,
+  };
+
+  await fxa.setSignedInUser(user);
+  let key = await fxa.keys.getKeyForScope(SCOPE_APP_SYNC);
+  Assert.deepEqual(key, {
+    scope: SCOPE_APP_SYNC,
+    ...MOCK_ACCOUNT_KEYS.scopedKeys[SCOPE_APP_SYNC],
+  });
+});
+
 add_task(async function test_getScopedKeys_unavailable_scope() {
   let fxa = new MockFxAccounts();
   let user = {
@@ -943,18 +948,39 @@ add_task(async function test_getScopedKeys_misconfigured_fxa_server() {
     ...getTestUser("eusebius"),
     uid: "aeaa1725c7a24ff983c6295725d5fc9b",
     verified: true,
-    kSync:
-      "0d6fe59791b05fa489e463ea25502e3143f6b7a903aa152e95cd9c6eddbac5b4dc68a19097ef65dbd147010ee45222444e66b8b3d7c8a441ebb7dd3dce015a9e",
-    kXCS: "22a42fe289dced5715135913424cb23b",
-    kExtSync:
-      "baded53eb3587d7900e604e8a68d860abf9de30b5c955d3c4d5dba63f26fd88265cd85923f6e9dcd16aef3b82bc88039a89c59ecd9e88de09a7418c7d94f90c9",
-    kExtKbHash:
-      "b776a89db29f22daedd154b44ff88397d0b210223fb956f5a749521dd8de8ddf",
+    keyFetchToken:
+      "808182838485868788898a8b8c8d8e8f909192939495969798999a9b9c9d9e9f",
+    unwrapBKey:
+      "6ea660be9c89ec355397f89afb282ea0bf21095760c8c5009bbcc894155bbe2a",
+    sessionToken: "mock session token, used in metadata request",
   };
   await fxa.setSignedInUser(user);
   await Assert.rejects(
-    fxa.keys.getKeyForScope(SCOPE_OLD_SYNC),
-    /The FxA server did not grant Firefox the `oldsync` scope/
+    fxa.keys.getKeyForScope(SCOPE_APP_SYNC),
+    /The FxA server did not grant Firefox the sync scope/
+  );
+});
+
+add_task(async function test_setScopedKeys() {
+  const fxa = new MockFxAccounts();
+  const user = {
+    ...getTestUser("foo"),
+    verified: true,
+  };
+  await fxa.setSignedInUser(user);
+  await fxa.keys.setScopedKeys(MOCK_ACCOUNT_KEYS.scopedKeys);
+  const key = await fxa.keys.getKeyForScope(SCOPE_APP_SYNC);
+  Assert.deepEqual(key, {
+    scope: SCOPE_APP_SYNC,
+    ...MOCK_ACCOUNT_KEYS.scopedKeys[SCOPE_APP_SYNC],
+  });
+});
+
+add_task(async function test_setScopedKeys_user_not_signed_in() {
+  const fxa = new MockFxAccounts();
+  await Assert.rejects(
+    fxa.keys.setScopedKeys(MOCK_ACCOUNT_KEYS.scopedKeys),
+    /Cannot persist keys, no user signed in/
   );
 });
 
@@ -967,17 +993,17 @@ add_test(function test_fetchAndUnwrapAndDeriveKeys_no_token() {
 
   makeObserver(ONLOGOUT_NOTIFICATION, function () {
     log.debug("test_fetchAndUnwrapKeys_no_token observed logout");
-    fxa._internal.getUserAccountData().then(user2 => {
+    fxa._internal.getUserAccountData().then(() => {
       fxa._internal.abortExistingFlow().then(run_next_test);
     });
   });
 
   fxa
     .setSignedInUser(user)
-    .then(user2 => {
+    .then(() => {
       return fxa.keys._fetchAndUnwrapAndDeriveKeys();
     })
-    .catch(error => {
+    .catch(() => {
       log.info("setSignedInUser correctly rejected");
     });
 });
@@ -986,6 +1012,7 @@ add_test(function test_fetchAndUnwrapAndDeriveKeys_no_token() {
 // signs in with a verified email.  Ensure that no sign-in events are triggered
 // on Alice's behalf.  In the end, Bob should be the signed-in user.
 add_test(function test_overlapping_signins() {
+  ensureOauthNotConfigured();
   let fxa = new MockFxAccounts();
   let alice = getTestUser("alice");
   let bob = getTestUser("bob");
@@ -1138,7 +1165,7 @@ add_test(function test_resend_email() {
   });
 });
 
-Services.prefs.setCharPref(
+Services.prefs.setStringPref(
   "identity.fxaccounts.remote.oauth.uri",
   "https://example.com/v1"
 );
@@ -1162,7 +1189,10 @@ add_test(async function test_getOAuthTokenWithSessionToken() {
   ) => {
     oauthTokenCalled = true;
     Assert.equal(sessionTokenHex, "alice's session token");
-    Assert.equal(clientId, "5882386c6d801776");
+    Assert.equal(
+      clientId,
+      CLIENT_IS_THUNDERBIRD ? "8269bacd7bbc7f80" : "5882386c6d801776"
+    );
     Assert.equal(scope, "profile");
     Assert.equal(ttl, undefined);
     return MOCK_TOKEN_RESPONSE;
@@ -1268,11 +1298,7 @@ add_task(async function test_getOAuthTokenCachedScopeNormalization() {
   let numOAuthTokenCalls = 0;
 
   let client = fxa._internal.fxAccountsClient;
-  client.accessTokenWithSessionToken = async (
-    _sessionTokenHex,
-    _clientId,
-    scopeString
-  ) => {
+  client.accessTokenWithSessionToken = async (_sessionTokenHex, _clientId) => {
     numOAuthTokenCalls++;
     return MOCK_TOKEN_RESPONSE;
   };
@@ -1341,7 +1367,7 @@ add_test(function test_getOAuthToken_invalid_scope_array() {
 add_test(function test_getOAuthToken_misconfigure_oauth_uri() {
   let fxa = new MockFxAccounts();
 
-  const prevServerURL = Services.prefs.getCharPref(
+  const prevServerURL = Services.prefs.getStringPref(
     "identity.fxaccounts.remote.oauth.uri"
   );
   Services.prefs.deleteBranch("identity.fxaccounts.remote.oauth.uri");
@@ -1349,7 +1375,7 @@ add_test(function test_getOAuthToken_misconfigure_oauth_uri() {
   fxa.getOAuthToken().catch(err => {
     Assert.equal(err.message, "INVALID_PARAMETER");
     // revert the pref
-    Services.prefs.setCharPref(
+    Services.prefs.setStringPref(
       "identity.fxaccounts.remote.oauth.uri",
       prevServerURL
     );
@@ -1400,6 +1426,31 @@ add_test(function test_getOAuthToken_error() {
   });
 });
 
+add_test(async function test_getOAuthTokenAndKey_errors_if_user_change() {
+  const fxa = new MockFxAccounts();
+  const alice = getTestUser("alice");
+  const bob = getTestUser("bob");
+  alice.verified = true;
+  bob.verified = true;
+
+  fxa.getOAuthToken = async () => {
+    // We mock what would happen if the user got changed
+    // after we got the access token
+    await fxa.setSignedInUser(bob);
+    return "access token";
+  };
+  fxa.keys.getKeyForScope = () => Promise.resolve("key!");
+  await fxa.setSignedInUser(alice);
+  await Assert.rejects(
+    fxa.getOAuthTokenAndKey({ scope: "foo", ttl: 10 }),
+    err => {
+      Assert.equal(err.message, ERROR_INVALID_ACCOUNT_STATE);
+      return true; // expected error
+    }
+  );
+  run_next_test();
+});
+
 add_task(async function test_listAttachedOAuthClients() {
   const ONE_HOUR = 60 * 60 * 1000;
   const ONE_DAY = 24 * ONE_HOUR;
@@ -1420,7 +1471,7 @@ add_task(async function test_listAttachedOAuthClients() {
           deviceId: "deadbeef",
           sessionTokenId: null,
           name: "Firefox Preview (no session token)",
-          scope: ["profile", "https://identity.mozilla.com/apps/oldsync"],
+          scope: ["profile", SCOPE_APP_SYNC],
           lastAccessTime: Date.now(),
         },
         {
@@ -1486,6 +1537,87 @@ add_task(async function test_listAttachedOAuthClients() {
   ]);
 });
 
+add_task(async function test_listAttachedOAuthClients_withCaching() {
+  const ONE_HOUR = 60 * 60 * 1000;
+  const ONE_DAY = 24 * ONE_HOUR;
+
+  const timestamp = Date.now();
+
+  let fxa = new MockFxAccounts();
+  let alice = getTestUser("alice");
+  alice.verified = true;
+
+  let client = fxa._internal.fxAccountsClient;
+  let originalResponse = {
+    body: [
+      {
+        clientId: "a2270f727f45f648",
+        deviceId: "deadbeef",
+        sessionTokenId: null,
+        name: "Firefox Preview (no session token)",
+        scope: ["profile", SCOPE_APP_SYNC],
+        lastAccessTime: Date.now(),
+      },
+      {
+        clientId: "802d56ef2a9af9fa",
+        deviceId: null,
+        sessionTokenId: null,
+        name: "Firefox Monitor",
+        scope: ["profile"],
+        lastAccessTime: Date.now() - ONE_DAY - ONE_HOUR,
+      },
+    ],
+    headers: { "x-timestamp": timestamp.toString() },
+  };
+
+  // Mock the client method.
+  client.attachedClients = async () => {
+    return originalResponse;
+  };
+
+  await fxa.setSignedInUser(alice);
+
+  // First call: should fetch from server
+  const clientsFirstCall = await fxa.listAttachedOAuthClients();
+  Assert.deepEqual(clientsFirstCall, [
+    { id: "a2270f727f45f648", lastAccessedDaysAgo: 0 },
+    { id: "802d56ef2a9af9fa", lastAccessedDaysAgo: 1 },
+  ]);
+
+  // Now modify the client so if it calls again, we would get different clients.
+  const updatedResponse = {
+    body: [
+      {
+        clientId: "updated-client",
+        lastAccessTime: Date.now() - ONE_DAY * 3,
+      },
+    ],
+    headers: { "x-timestamp": (timestamp + 1000).toString() },
+  };
+  client.attachedClients = async () => {
+    return updatedResponse;
+  };
+
+  // Second call without forceRefresh: should return cached data, not the updated one.
+  const clientsSecondCall = await fxa.listAttachedOAuthClients();
+  Assert.deepEqual(
+    clientsSecondCall,
+    [
+      { id: "a2270f727f45f648", lastAccessedDaysAgo: 0 },
+      { id: "802d56ef2a9af9fa", lastAccessedDaysAgo: 1 },
+    ],
+    "Should return cached clients from the first call, ignoring the updated mock"
+  );
+
+  // Now force refresh
+  const clientsForceRefresh = await fxa.listAttachedOAuthClients(true);
+  Assert.deepEqual(
+    clientsForceRefresh,
+    [{ id: "updated-client", lastAccessedDaysAgo: 3 }],
+    "Forcing a refresh should return the updated data"
+  );
+});
+
 add_task(async function test_getSignedInUserProfile() {
   let alice = getTestUser("alice");
   alice.verified = true;
@@ -1567,35 +1699,6 @@ add_task(async function test_checkVerificationStatusFailed() {
   Assert.equal(user.sessionToken, null);
 });
 
-add_task(async function test_deriveKeys() {
-  let account = await MakeFxAccounts();
-  let kBhex =
-    "fd5c747806c07ce0b9d69dcfea144663e630b65ec4963596a22f24910d7dd15d";
-  let kB = CommonUtils.hexToBytes(kBhex);
-  const uid = "1ad7f502-4cc7-4ec1-a209-071fd2fae348";
-
-  const { kSync, kXCS, kExtSync, kExtKbHash } = await account.keys._deriveKeys(
-    uid,
-    kB
-  );
-
-  Assert.equal(
-    kSync,
-    "ad501a50561be52b008878b2e0d8a73357778a712255f7722f497b5d4df14b05" +
-      "dc06afb836e1521e882f521eb34691d172337accdbf6e2a5b968b05a7bbb9885"
-  );
-  Assert.equal(kXCS, "6ae94683571c7a7c54dab4700aa3995f");
-  Assert.equal(
-    kExtSync,
-    "f5ccd9cfdefd9b1ac4d02c56964f59239d8dfa1ca326e63696982765c1352cdc" +
-      "5d78a5a9c633a6d25edfea0a6c221a3480332a49fd866f311c2e3508ddd07395"
-  );
-  Assert.equal(
-    kExtKbHash,
-    "6192f1cc7dce95334455ba135fa1d8fca8f70e8f594ae318528de06f24ed0273"
-  );
-});
-
 add_task(async function test_flushLogFile() {
   _("Tests flushLogFile");
   let account = await MakeFxAccounts();
